import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { UserModule } from './modules/user/user.module';
import {
  HealthModule,
  HealthModuleOptions,
  HealthRepositoryPort,
} from '@mobicoop/health-module';
import { MessagerModule } from './modules/messager/messager.module';
import { USER_REPOSITORY } from './modules/user/user.di-tokens';
import { MESSAGE_PUBLISHER } from './modules/messager/messager.di-tokens';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { EventEmitterModule } from '@nestjs/event-emitter';
import {
  HEALTH_CRITICAL_LOGGING_KEY,
  HEALTH_USER_REPOSITORY,
  SERVICE_NAME,
} from './app.constants';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    EventEmitterModule.forRoot(),
    HealthModule.forRootAsync({
      imports: [UserModule, MessagerModule],
      inject: [USER_REPOSITORY, MESSAGE_PUBLISHER],
      useFactory: async (
        userRepository: HealthRepositoryPort,
        messagePublisher: MessagePublisherPort,
      ): Promise<HealthModuleOptions> => ({
        serviceName: SERVICE_NAME,
        criticalLoggingKey: HEALTH_CRITICAL_LOGGING_KEY,
        checkRepositories: [
          {
            name: HEALTH_USER_REPOSITORY,
            repository: userRepository,
          },
        ],
        messagePublisher,
      }),
    }),
    UserModule,
    MessagerModule,
  ],
  exports: [UserModule, MessagerModule],
})
export class AppModule {}
