import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { UserMapper } from '@modules/user/user.mapper';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { FindUsersByIdsRequestDto } from './dtos/find-users-by-ids.request.dto';
import { FindUsersByIdsQuery } from '@modules/user/core/application/queries/find-users-by-ids/find-users-by-ids.query';
import { UsersResponseDto } from '../dtos/users.response.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class FindUsersByIdsGrpcController {
  constructor(
    protected readonly mapper: UserMapper,
    private readonly queryBus: QueryBus,
  ) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'FindAllByIds')
  async findAllbyIds(
    data: FindUsersByIdsRequestDto,
  ): Promise<UsersResponseDto> {
    try {
      const users: UserEntity[] = await this.queryBus.execute(
        new FindUsersByIdsQuery(data.ids),
      );
      return {
        users: users.map((user: UserEntity) => this.mapper.toResponse(user)),
      };
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
