import { Test, TestingModule } from '@nestjs/testing';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import { USER_REPOSITORY } from '@modules/user/user.di-tokens';
import { FindUsersByIdsQueryHandler } from '@modules/user/core/application/queries/find-users-by-ids/find-users-by-ids.query-handler';
import { FindUsersByIdsQuery } from '@modules/user/core/application/queries/find-users-by-ids/find-users-by-ids.query';

const now = new Date('2023-06-21 06:00:00');
const user1: UserEntity = new UserEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    firstName: 'John',
    lastName: 'Doe',
    email: 'john.doe@email.com',
    phone: '+33611223344',
  },
  createdAt: now,
  updatedAt: now,
});
const user2: UserEntity = new UserEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df55',
  props: {
    firstName: 'Jane',
    lastName: 'Doe',
    email: 'jane.doe@email.com',
    phone: '+33611223355',
  },
  createdAt: now,
  updatedAt: now,
});
const user3: UserEntity = new UserEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df66',
  props: {
    firstName: 'Jimmy',
    lastName: 'Doe',
    email: 'jimmy.doe@email.com',
    phone: '+33611223366',
  },
  createdAt: now,
  updatedAt: now,
});

const mockUserRepository = {
  findAllByIds: jest.fn().mockImplementation(() => [user1, user2, user3]),
};

describe('Find Users By Ids Query Handler', () => {
  let findUsersByIdsQueryHandler: FindUsersByIdsQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_REPOSITORY,
          useValue: mockUserRepository,
        },
        FindUsersByIdsQueryHandler,
      ],
    }).compile();

    findUsersByIdsQueryHandler = module.get<FindUsersByIdsQueryHandler>(
      FindUsersByIdsQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(findUsersByIdsQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return users', async () => {
      const findUsersByIdsQuery = new FindUsersByIdsQuery([
        'c160cf8c-f057-4962-841f-3ad68346df44',
        'c160cf8c-f057-4962-841f-3ad68346df55',
        'c160cf8c-f057-4962-841f-3ad68346df66',
      ]);
      const users: UserEntity[] =
        await findUsersByIdsQueryHandler.execute(findUsersByIdsQuery);
      expect(users).toHaveLength(3);
      expect(users[1].getProps().firstName).toBe('Jane');
    });
  });
});
