import { Test, TestingModule } from '@nestjs/testing';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { PublishMessageWhenUserIsDeletedDomainEventHandler } from '@modules/user/core/application/event-handlers/publish-message-when-user-is-deleted.domain-event-handler';
import { UserDeletedDomainEvent } from '@modules/user/core/domain/events/user-deleted.domain-event';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when user is deleted domain event handler', () => {
  let publishMessageWhenUserIsDeletedDomainEventHandler: PublishMessageWhenUserIsDeletedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: USER_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenUserIsDeletedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenUserIsDeletedDomainEventHandler =
      module.get<PublishMessageWhenUserIsDeletedDomainEventHandler>(
        PublishMessageWhenUserIsDeletedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const userDeletedDomainEvent: UserDeletedDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenUserIsDeletedDomainEventHandler.handle(
      userDeletedDomainEvent,
    );
    expect(publishMessageWhenUserIsDeletedDomainEventHandler).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      'user.deleted',
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000}}',
    );
  });
});
