import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { FindUsersByIdsGrpcController } from '@modules/user/interface/grpc-controllers/find-users-by-ids.grpc.controller';
import { UserMapper } from '@modules/user/user.mapper';
import { QueryBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const mockQueryBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => [
      '6dcf093c-c7db-4dae-8e9c-c715cebf83c7',
      '6dcf093c-c7db-4dae-8e9c-c715cebf83c8',
      '6dcf093c-c7db-4dae-8e9c-c715cebf83c9',
    ])
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockUserMapper = {
  toResponse: jest.fn().mockImplementationOnce(() => ({
    firstName: 'John',
    lastName: 'Doe',
    email: 'john.doe@email.com',
    phone: '+33611223344',
  })),
};

describe('Find Users By Ids Grpc Controller', () => {
  let findUsersbyIdsGrpcController: FindUsersByIdsGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: QueryBus,
          useValue: mockQueryBus,
        },
        {
          provide: UserMapper,
          useValue: mockUserMapper,
        },
        FindUsersByIdsGrpcController,
      ],
    }).compile();

    findUsersbyIdsGrpcController = module.get<FindUsersByIdsGrpcController>(
      FindUsersByIdsGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(findUsersbyIdsGrpcController).toBeDefined();
  });

  it('should return users', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockUserMapper, 'toResponse');
    const response = await findUsersbyIdsGrpcController.findAllbyIds({
      ids: [
        '6dcf093c-c7db-4dae-8e9c-c715cebf83c7',
        '6dcf093c-c7db-4dae-8e9c-c715cebf83c8',
        '6dcf093c-c7db-4dae-8e9c-c715cebf83c9',
      ],
    });
    expect(response.users).toHaveLength(3);
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockUserMapper.toResponse).toHaveBeenCalledTimes(3);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockUserMapper, 'toResponse');
    expect.assertions(4);
    try {
      await findUsersbyIdsGrpcController.findAllbyIds({
        ids: [
          '6dcf093c-c7db-4dae-8e9c-c715cebf83c7',
          '6dcf093c-c7db-4dae-8e9c-c715cebf83c8',
          '6dcf093c-c7db-4dae-8e9c-c715cebf83c9',
        ],
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockUserMapper.toResponse).toHaveBeenCalledTimes(0);
  });
});
