import { UserEntity } from '@modules/user/core/domain/user.entity';
import {
  UserReadModel,
  UserWriteModel,
} from '@modules/user/infrastructure/user.repository';
import { UserResponseDto } from '@modules/user/interface/dtos/user.response.dto';
import { UserMapper } from '@modules/user/user.mapper';
import { Test } from '@nestjs/testing';

const now = new Date('2023-06-21 06:00:00');
const userEntity: UserEntity = new UserEntity({
  id: 'c160cf8c-f057-4962-841f-3ad68346df44',
  props: {
    firstName: 'john',
    lastName: 'doe',
    email: 'john.doe@email.com',
    phone: '+33611223344',
  },
  createdAt: now,
  updatedAt: now,
});
const userReadModel: UserReadModel = {
  uuid: 'c160cf8c-f057-4962-841f-3ad68346df44',
  firstName: 'John',
  lastName: 'Doe',
  email: 'john.doe@email.com',
  phone: '+33611223344',
  createdAt: now,
  updatedAt: now,
};

describe('User Mapper', () => {
  let userMapper: UserMapper;

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      providers: [UserMapper],
    }).compile();
    userMapper = module.get<UserMapper>(UserMapper);
  });

  it('should be defined', () => {
    expect(userMapper).toBeDefined();
  });

  it('should map domain entity to persistence data', async () => {
    const mapped: UserWriteModel = userMapper.toPersistence(userEntity);
    expect(mapped.lastName).toBe('doe');
  });

  it('should map persisted data to domain entity', async () => {
    const mapped: UserEntity = userMapper.toDomain(userReadModel);
    expect(mapped.getProps().firstName).toBe('John');
  });

  it('should map domain entity to response', async () => {
    const mapped: UserResponseDto = userMapper.toResponse(userEntity);
    expect(mapped.id).toBe('c160cf8c-f057-4962-841f-3ad68346df44');
  });
});
